#!/bin/bash
#
# $Id: fwipsec.sh,v 1.5 2004/05/09 17:13:49 jay Exp $
#
#
# Description: Ye Olde El Mero Chingon script for fwipsec aka "the guts"
#
# Written by:
# Jay Pfeifer (pfeifer@gentoo.org)
# Mark Seamans (mark@seamans.cc)
#
# Copyright 1998-2004: Jay Pfeifer & Mark Seamans
#
#############################################################################
# Initialize Firewall Configuration
#############################################################################

#-- Set path for script
PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin

#-- Bring in configuration
. /etc/fwipsec/fwipsec.conf

#############################################################################
# Create FreeS/WAN config files
#############################################################################
# Removed until I (jay) get it "friendlier" with 2+ VPNs

#############################################################################
# Create squid/squidGuard config files
#############################################################################
# Removed until I (jay) get it "friendlier" with 2+ netblocks

#############################################################################
# Start FW Setup
#############################################################################
echo "Start Firewall Rule Setup"

#############################################################################
# Turn on forwarding and dynamic addressing.
#############################################################################
echo "Enable Forwarding and Dynamic Addressing"
echo 1 > /proc/sys/net/ipv4/ip_forward
echo 1 > /proc/sys/net/ipv4/ip_dynaddr

#############################################################################
# Turn off anti-spoofing (aka keep freeswan from bitching).
#############################################################################
echo "Setting rp_filter for FreeS/WAN"
for f in /proc/sys/net/ipv4/conf/*/rp_filter #*/
do
  echo 0 > ${f}
done

#############################################################################
# Enable broadcast echo Protection
#############################################################################
echo "Enabling Broadcast Echo Protection"
echo 1 > /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts

#############################################################################
# Disable Source Routed Packets
#############################################################################
echo "Disabling IP Source Routing of Packets"
for f in /proc/sys/net/ipv4/conf/*/accept_source_route #*/
do
  echo 0 > ${f}
done

#############################################################################
# Enable TCP SYN Cookie Protection
#############################################################################
echo "Enabling TCP SYN Cookie Protection"
echo 1 > /proc/sys/net/ipv4/tcp_syncookies

#############################################################################
# Disable ICMP Redirects
#############################################################################
echo "Disabling ICMP Redirect acceptance"
for f in /proc/sys/net/ipv4/conf/*/accept_redirects #*/
do
  echo 0 > ${f}
done

#############################################################################
# Turn-off ip redirects
#############################################################################
echo "Turn-off sending of ip redirects"
for f in /proc/sys/net/ipv4/conf/*/send_redirects #*/
do
  echo 0 > ${f}
done

#############################################################################
# Log packets having impossible addresses.
#############################################################################
echo "Enabling logging of packets having impossible addresses."
for f in /proc/sys/net/ipv4/conf/*/log_martians #*/
do
  echo 1 > ${f}
done

#############################################################################
# Flush all rulesets.
#############################################################################
echo "Flush All Rulesets"
${IPTABLES} -F INPUT
${IPTABLES} -F OUTPUT
${IPTABLES} -F FORWARD
${IPTABLES} -X

#############################################################################
# Load ip_nat_ftp based upon user setting 
#############################################################################
case ${NATFTP} in
yes|YES)
	echo "FTP NAT is set to ${NATFTP}"
	echo "This will allow NAT'd active ftp sessions."
	${LSMOD} | grep ${MODNATFTP} || ${MODPROBE} ${MODNATFTP} 2>/dev/null 
	;;
no|NO)
	echo "FTP NAT is set to ${NATFTP}"
	echo "This will disallow NAT'd active ftp sessions."
	${LSMOD} | grep ${MODNATFTP} && ${RMMOD} ${MODNATFTP} 2>/dev/null 
	;;
esac

#############################################################################
# Local Interface & Netblock Info
#############################################################################
echo "External interface is ${PUBIF} with IP ${PUBIF_IP}"
echo "Internal interface is ${INTIF} with IP ${INTIF_IP}"
echo "Internal network is ${INTIF_NET}"
echo "Gateway IP is ${GATEWAY} with network of ${PUBIF_NET}"

#############################################################################
# Allow loopback interface
#############################################################################
echo "Allow loopback"
${IPTABLES} -A INPUT -i lo -s ${ALL} -d ${ALL} -j ACCEPT
${IPTABLES} -A OUTPUT -o lo -s ${ALL} -d ${ALL} -j ACCEPT

#############################################################################
# Set Logging
#############################################################################
#Log Invalid
${IPTABLES} -t mangle -A PREROUTING -j LOG -m state --state INVALID --log-prefix "[FWIPSEC] (INVALID drop) "

#############################################################################
# Allow Internal Network to access and be forwarded/nat'd
#############################################################################
echo "Allow internal network (${INTIF_NET}) access"
${IPTABLES} -A INPUT  -s ${INTIF_NET} -d ${ALL} -j ACCEPT
${IPTABLES} -A FORWARD  -s ${INTIF_NET} -d ${ALL} -j ACCEPT

#############################################################################
# Masquerading
#############################################################################
echo "Kill all masquerading (NAT)"

#----------------------------------------------------------#
#     Turn off masquerading within masqueraded networks.
#----------------------------------------------------------#

${IPTABLES} -t nat -A POSTROUTING -s ${INTIF_NET} -d ${INTIF_NET} -j ACCEPT

for x in $(seq $(grep -c $ ${FWDEFS}))
do
x=$(($x))
line="$(sed -n ${x}p ${FWDEFS})"
SERVICETYPE=`echo ${line} | awk '{print $1}'`

case ${SERVICETYPE} in
vpn|VPN)
	VPN_NET=`echo ${line} | awk '{print $3}'`
	${IPTABLES} -t nat -A POSTROUTING -s ${INTIF_NET} -d ${VPN_NET} -j ACCEPT
	${IPTABLES} -t nat -A POSTROUTING -s ${VPN_NET} -d ${INTIF_NET} -j ACCEPT
	;;
esac
done

${IPTABLES} -t nat -A POSTROUTING -s ${INTIF_NET} -d ${LO_NET} -j ACCEPT
${IPTABLES} -t nat -A POSTROUTING -s ${LO_NET} -d ${INTIF_NET} -j ACCEPT
${IPTABLES} -t nat -A POSTROUTING -s ${LO_NET} -d ${LO_NET} -j ACCEPT

#----------------------------------------------------------#
#     Turn off masquerading of the external interface.
#----------------------------------------------------------#

${IPTABLES} -t nat -A POSTROUTING -s ${PUBIF_NET} -d ${ALL} -j ACCEPT

#----------------------------------------------------------#
#     Set static mappings.
#----------------------------------------------------------#

for x in $(seq $(grep -c $ ${FWDEFS}))
do
x=$(($x))
line="$(sed -n ${x}p ${FWDEFS})"
SERVICETYPE=`echo ${line} | awk '{print $1}'`

case ${SERVICETYPE} in
static|STATIC)
	PUB_IP=`echo ${line} | awk '{print $2}'`
	LOCAL_HOST=`echo ${line} | awk '{print $3}'`
	echo -e "Setting static mapping of public ip ${PUB_IP} to private ip ${LOCAL_HOST}"
	${IPADD} ${PUB_IP} dev ${PUBIF} 2>/dev/null
	${IPTABLES} -t nat -A POSTROUTING -o ${PUBIF} --src ${LOCAL_HOST} -j SNAT --to ${PUB_IP}
	${IPTABLES} -t nat -A PREROUTING -d ${PUB_IP} -j DNAT --to ${LOCAL_HOST}
	;;
addip|ADDIP)
	ADDIP_INTERFACE=`echo ${line} | awk '{print $2}'`
	PUB_IP=`echo ${line} | awk '{print $3}'`
	echo -e "Adding IP Address ${PUB_IP} to ${ADDIP_INTERFACE}"
	${IPADD} ${PUB_IP} dev ${ADDIP_INTERFACE} 2>/dev/null
	;;
esac
done

#----------------------------------------------------------#
#     Allow various DHCPD's to function.
#----------------------------------------------------------#
for x in $(seq $(grep -c $ ${FWDEFS}))
do
x=$(($x))
line="$(sed -n ${x}p ${FWDEFS})"
SERVICETYPE=`echo ${line} | awk '{print $1}'`
case ${SERVICETYPE} in
dhcpd|DHCPD)
	DHCPD_IF=`echo ${line} | awk '{print $2}'`
	echo "Allowing DHCP broadcast requests in ${DHCPD_IF}"
	${IPTABLES} -A INPUT -i ${DHCPD_IF} -p udp --dport 67 -d ${DHCPTARGET} --sport 68 -j ACCEPT
	;;
esac
done


#----------------------------------------------------------#
#     Masquerade all other networks.
#----------------------------------------------------------#

case ${NAT} in
yes|YES)
	echo "Masquerading for default network is set to ${NAT}"
	echo "Internal network WILL be masqueraded by default"
	echo "Masquerading ${INTIF_NET} on ${PUBIF} with ip address ${PUBIF_IP}"
	${IPTABLES} -t nat -A POSTROUTING -s ${INTIF_NET} -d ${ALL} -j MASQUERADE
	;;
no|NO)
	echo "Masquerading for default network is set to ${NAT}"
	echo "Internal network *WILL NOT* be masqueraded by default"
	;;
esac

${IPTABLES} -t nat -A POSTROUTING -s ${LO_NET} -d ${ALL} -j MASQUERADE

# Below subroutine masquerades other defined net(s) in fwipsec.defs
for x in $(seq $(grep -c $ ${FWDEFS}))
do
x=$(($x))
line="$(sed -n ${x}p ${FWDEFS})"
SERVICETYPE=`echo ${line} | awk '{print $1}'`
case ${SERVICETYPE} in
masq|MASQ)
	MASQ_NET=`echo ${line} | awk '{print $2}'`
	echo "Masquerading ${MASQ_NET} on ${PUBIF} with ip address ${PUBIF_IP}"
	${IPTABLES} -A INPUT  -s ${MASQ_NET} -d $ALL -j ACCEPT
	${IPTABLES} -A FORWARD  -s ${MASQ_NET} -d $ALL -j ACCEPT
	${IPTABLES} -t nat -A POSTROUTING -s ${MASQ_NET} -d ${ALL} -j MASQUERADE
	;;
esac
done

############################################################
# Input/Forward/Output Firewall Rules                      #
############################################################
echo "Establish Firewall Input/Forward/Output Rules"
${IPTABLES} -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
${IPTABLES} -A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT
${IPTABLES} -A OUTPUT -m state --state ESTABLISHED,RELATED -j ACCEPT

############################################################
# New subroutine for setting up custom services...         #
############################################################
# Count the number of lines in "fwipsec.defs"
# Then run a loop that number of times
# Somehow, I could make this much more efficient...
# But it works, and it doesn't seem too slow... so this low on the priority

for x in $(seq $(grep -c $ ${FWDEFS}))
do
x=$(($x))

line="$(sed -n ${x}p ${FWDEFS})"

SERVICETYPE=`echo ${line} | awk '{print $1}'`

case ${SERVICETYPE} in

vpn|VPN)
	VPN_PEER=`echo ${line} | awk '{print $2}'`
	VPN_NET=`echo ${line} | awk '{print $3}'`
	echo -e "Setting up VPN access from ${VPN_PEER} with remote netblock ${VPN_NET}"
	${IPTABLES} -A FORWARD  -p all -s ${VPN_NET} -d ${INTIF_NET} -j ACCEPT
	${IPTABLES} -A FORWARD  -p all -s ${INTIF_NET} -d ${VPN_NET} -j ACCEPT
	${IPTABLES} -A INPUT  -p all -s ${VPN_NET} -d ${INTIF_IP} -j ACCEPT
	${IPTABLES} -A INPUT  -p udp -s ${VPN_PEER} -d ${PUBIF_IP} --dport 500 -j ACCEPT
	${IPTABLES} -A OUTPUT  -p udp -s ${PUBIF_IP} -d ${VPN_PEER} --dport 500 -j ACCEPT
	${IPTABLES} -A INPUT  -p esp -s ${VPN_PEER} -d ${PUBIF_IP} -j ACCEPT
	${IPTABLES} -A OUTPUT  -p esp -s ${PUBIF_IP} -d ${VPN_PEER} -j ACCEPT
	${IPTABLES} -A INPUT  -p ah -s ${VPN_PEER} -d ${PUBIF_IP} -j ACCEPT
	${IPTABLES} -A OUTPUT  -p ah -s ${PUBIF_IP} -d ${VPN_PEER} -j ACCEPT
	${IPTABLES} -A INPUT  -p icmp -s ${VPN_PEER} -d ${PUBIF_IP} -j ACCEPT
	${IPTABLES} -A INPUT  -p icmp -s ${VPN_PEER} --icmp-type 8 -d ${PUBIF_IP} -j ACCEPT
	${IPTABLES} -A OUTPUT  -p icmp -s ${PUBIF_IP} -d ${VPN_PEER} -j ACCEPT
	${IPTABLES} -A OUTPUT  -p icmp -s ${PUBIF_IP} --icmp-type 8 -d ${VPN_PEER} -j ACCEPT
	;;
static|STATIC)
#	PUB_IP=`echo ${line} | awk '{print $2}'`
#	LOCAL_HOST=`echo ${line} | awk '{print $3}'`
	;;
addip|ADDIP)
#	PUB_IP=`echo ${line} | awk '{print $2}'`
#	LOCAL_HOST=`echo ${line} | awk '{print $3}'`
	;;
dhcpd|DHCPD)
#	PUB_IP=`echo ${line} | awk '{print $2}'`
#	LOCAL_HOST=`echo ${line} | awk '{print $3}'`
	;;
ftp|FTP)
	FTP_SERVICE=`echo ${line} | awk '{print $2}'`
	case ${FTP_SERVICE} in
	alocal|ALOCAL)
	APUBIF_IP=`echo ${line} | awk '{print $3}'`
	FTP_ACCESS=`echo ${line} | awk '{print $4}'`
	echo -e "Setting up ftp access to ${APUBIF_IP} for netblock ${FTP_ACCESS}"
	${IPTABLES} -A INPUT -p tcp -s ${FTP_ACCESS} -d ${APUBIF_IP} --dport 21 -j ACCEPT
	# Allow incoming connections to ftp server on port 21
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${APUBIF_IP} --dport 21 -m state --state NEW -j ACCEPT
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${APUBIF_IP} --dport 21 -j ACCEPT
	${IPTABLES} -A OUTPUT -o ${PUBIF} -p tcp ! --syn -s ${APUBIF_IP} --sport 21 --dport ${HIPORTS} -j ACCEPT
	# Allow passive mode FTP data channel responses
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${APUBIF_IP} --dport ${HIPORTS} -m state --state NEW -j ACCEPT
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${APUBIF_IP} --dport ${HIPORTS} -j ACCEPT
	${IPTABLES} -A OUTPUT -o $PUBIF -p tcp ! --syn -s ${APUBIF_IP} --sport ${HIPORTS} --dport ${HIPORTS} -j ACCEPT
	;;
	local|LOCAL)
	FTP_ACCESS=`echo ${line} | awk '{print $3}'`
	echo -e "Setting up ftp access to ${PUBIF_IP} for netblock ${FTP_ACCESS}"
	${IPTABLES} -A INPUT -p tcp -s ${FTP_ACCESS} -d ${PUBIF_IP} --dport 21 -j ACCEPT
	# Allow incoming connections to ftp server on port 21
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${PUBIF_IP} --dport 21 -m state --state NEW -j ACCEPT
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${PUBIF_IP} --dport 21 -j ACCEPT
	${IPTABLES} -A OUTPUT -o ${PUBIF} -p tcp ! --syn -s ${PUBIF_IP} --sport 21 --dport ${HIPORTS} -j ACCEPT
	# Allow passive mode FTP data channel responses
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${PUBIF_IP} --dport ${HIPORTS} -m state --state NEW -j ACCEPT
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${PUBIF_IP} --dport ${HIPORTS} -j ACCEPT
	${IPTABLES} -A OUTPUT -o ${PUBIF} -p tcp ! --syn -s ${PUBIF_IP} --sport ${HIPORTS} --dport ${HIPORTS} -j ACCEPT
	;;
	forward|FORWARD)
	FTP_PUB_IP=`echo ${line} | awk '{print $3}'`
	FTP_HOST=`echo ${line} | awk '{print $4}'`
	FTP_ACCESS=`echo ${line} | awk '{print $5}'`
	echo -e "Setting up ftp access to ${FTP_PUB_IP} for netblock ${FTP_ACCESS}"
	${IPTABLES} -I FORWARD -p tcp -d ${FTP_HOST} --dport 21 -j ACCEPT
	${IPTABLES} -t nat -A PREROUTING -p tcp -s ${FTP_ACCESS} -d ${FTP_PUB_IP} --dport 21 -j DNAT --to ${FTP_HOST}:21
	# Allow incoming connections to ftp server on port 21
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${FTP_PUB_IP} --dport 21 -m state --state NEW -j ACCEPT
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${FTP_PUB_IP} --dport 21 -j ACCEPT
	${IPTABLES} -A OUTPUT -o ${PUBIF} -p tcp ! --syn -s ${FTP_PUB_IP} --sport 21 --dport ${HIPORTS} -j ACCEPT
	# Allow passive mode FTP data channel responses
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${FTP_PUB_IP} --dport ${HIPORTS} -m state --state NEW -j ACCEPT
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport ${HIPORTS} -d ${FTP_PUB_IP} --dport ${HIPORTS} -j ACCEPT
	${IPTABLES} -A OUTPUT -o ${PUBIF} -p tcp ! --syn -s ${FTP_PUB_IP} --sport ${HIPORTS} --dport ${HIPORTS} -j ACCEPT
	;;
	*)
	esac
	;;
proxy|PROXY)
	PROXY_PORT=`echo ${line} | awk '{print $2}'`
	echo "Transparent Proxy Redirect enabled on port ${PROXY_PORT}"
	# Outgoing LAN web client traffic is redirected to the squid server running on the local host.
	${IPTABLES} -t nat -A PREROUTING -i ${INTIF} -p tcp -s ${INTIF_NET} --sport ${HIPORTS} --dport 80 -j REDIRECT --to-port ${PROXY_PORT}
	#The redirected packet will be delivered to the input queue associated with the network interface conn to the LAN on this host
	${IPTABLES} -A INPUT -i ${INTIF} -p tcp -s ${INTIF_NET} --sport ${HIPORTS} -d ${INTIF_IP} --dport ${PROXY_PORT} -m state --state NEW,ESTABLISHED,RELATED -j ACCEPT
	#Squid is a proxy as well as a web cache server, so web traffic isn't forwared.
	#Instead the squid server establishes connections with the remote web servers, acting as a client itself:
	${IPTABLES} -A OUTPUT -o ${PUBIF} -p tcp -s ${PUBIF_IP} --sport ${HIPORTS} --dport 80 -m state --state NEW,ESTABLISHED,RELATED -j ACCEPT
	${IPTABLES} -A INPUT -i ${PUBIF} -p tcp --sport 80 -d ${PUBIF_IP} --dport ${HIPORTS} -m state --state ESTABLISHED,RELATED -j ACCEPT
	#Squid responds as a server to the LAN client
	${IPTABLES} -A OUTPUT -o ${INTIF} -p tcp -s ${INTIF_NET} --sport 80 --dport ${HIPORTS} -m state --state ESTABLISHED,RELATED -j ACCEPT
	;;
prio|PRIO)
	PRIO_PORT=`echo $line | awk '{print $3}'`
	PRIO_PROTO=`echo $line | awk '{print $2}'`
	echo -e "Setting ${PRIO_PROTO} ${PRIO_PORT} priority on incoming and outgoing data"
	${IPTABLES} -A PREROUTING -t mangle -p ${PRIO_PROTO} --sport ${PRIO_PORT} -j TOS --set-tos Minimize-Delay
	${IPTABLES} -A OUTPUT -t mangle -p ${PRIO_PROTO} --dport ${PRIO_PORT} -j TOS --set-tos Minimize-Delay
	;;
other|OTHER)
	OTHER_SERVICE=`echo ${line} | awk '{print $2}'`
	case ${OTHER_SERVICE} in
	local|LOCAL)
	PROTOCOL=`echo ${line} | awk '{print $3}'`
	LOCAL_PORT=`echo ${line} | awk '{print $4}'`
	REMOTE_ACCESS=`echo ${line} | awk '{print $5}'`
	echo "${PROTOCOL} Port ${LOCAL_PORT} on ${PUBIF_IP} is accessible from ${REMOTE_ACCESS}"
	${IPTABLES} -A INPUT  -p ${PROTOCOL} -s ${REMOTE_ACCESS} -d ${PUBIF_IP} --dport ${LOCAL_PORT} -j ACCEPT
	;;
	alocal|ALOCAL)
	APUBIFIP=`echo ${line} | awk '{print $3}'`
	PROTOCOL=`echo ${line} | awk '{print $4}'`
	LOCAL_PORT=`echo ${line} | awk '{print $5}'`
	REMOTE_ACCESS=`echo ${line} | awk '{print $6}'`
	echo "${PROTOCOL} Port ${LOCAL_PORT} on addtional address ${APUBIFIP} is accessible from ${REMOTE_ACCESS}"
	${IPTABLES} -A INPUT  -p ${PROTOCOL} -s ${REMOTE_ACCESS} -d ${APUBIFIP} --dport ${LOCAL_PORT} -j ACCEPT
	;;
	flocal|FLOCAL)
	PROTOCOL=`echo ${line} | awk '{print $3}'`
	PUB_PORT=`echo ${line} | awk '{print $4}'`
	LOCAL_HOST=`echo ${line} | awk '{print $5}'`
	LOCAL_PORT=`echo ${line} | awk '{print $6}'`
	REMOTE_ACCESS=`echo ${line} | awk '{print $7}'`
	echo "${PROTOCOL} Port ${PUBIF_IP}:${PUB_PORT} accessible from ${REMOTE_ACCESS} and forwarded to ${LOCAL_HOST}:${LOCAL_PORT}"
	${IPTABLES} -I FORWARD -p ${PROTOCOL} -d ${LOCAL_HOST} --dport ${LOCAL_PORT} -j ACCEPT
	${IPTABLES} -t nat -A PREROUTING -p ${PROTOCOL} -s ${REMOTE_ACCESS} -d ${PUBIF_IP} --dport ${PUB_PORT} -j DNAT --to ${LOCAL_HOST}:${LOCAL_PORT}
	;;
	forward|FORWARD)
	PROTOCOL=`echo ${line} | awk '{print $3}'`
	PUB_IP=`echo ${line} | awk '{print $4}'`
	PUB_PORT=`echo ${line} | awk '{print $5}'`
	LOCAL_HOST=`echo ${line} | awk '{print $6}'`
	LOCAL_PORT=`echo ${line} | awk '{print $7}'`
	REMOTE_ACCESS=`echo $line | awk '{print $8}'`
	echo "${PROTOCOL} Port ${PUB_PORT} accessible from ${REMOTE_ACCESS} and forwarded to ${LOCAL_HOST}:${LOCAL_PORT}"
	${IPTABLES} -I FORWARD -p ${PROTOCOL} -d ${LOCAL_HOST} --dport ${LOCAL_PORT} -j ACCEPT
	${IPTABLES} -t nat -A PREROUTING -p ${PROTOCOL} -s ${REMOTE_ACCESS} -d ${PUB_IP} --dport ${PUB_PORT} -j DNAT --to ${LOCAL_HOST}:${LOCAL_PORT}
	;;
	iptype|IPTYPE)
	PROTOCOL=`echo ${line} | awk '{print $3}'`
	PUB_IP=`echo ${line} | awk '{print $4}'`
	LOCAL_HOST=`echo ${line} | awk '{print $5}'`
	REMOTE_ACCESS=`echo $line | awk '{print $6}'`
	echo "IP Type ${PROTOCOL} accessible from ${REMOTE_ACCESS} and forwarded to ${LOCAL_HOST}"
#	${IPTABLES} -A INPUT  -p ${PROTOCOL} -s ${REMOTE_ACCESS} -d ${PUB_IP} -j ACCEPT
#	${IPTABLES} -A OUTPUT  -p ${PROTOCOL} -s ${PUB_IP} -d ${REMOTE_ACCESS} -j ACCEPT
	${IPTABLES} -I FORWARD -p ${PROTOCOL} -d ${LOCAL_HOST} -j ACCEPT
	${IPTABLES} -t nat -A PREROUTING -s ${REMOTE_ACCESS} -d ${PUB_IP} -p ${PROTOCOL} -j DNAT --to ${LOCAL_HOST}
	;;
	*)
	echo "No Match"
	esac
	;;
block|BLOCK)
	BLOCK_SERVICE=`echo ${line} | awk '{print $2}'`
	case ${BLOCK_SERVICE} in
	local|LOCAL)
	PROTOCOL=`echo ${line} | awk '{print $3}'`
	BLOCK_SOURCE=`echo ${line} | awk '{print $4}'`
	BLOCK_DEST=`echo ${line} | awk '{print $5}'`
	BLOCK_DEST_PORT=`echo ${line} | awk '{print $6}'`
	echo "${BLOCK_SOURCE} is denied access to ${BLOCK_DEST} on ${PROTOCOL} port ${BLOCK_DEST_PORT}"
	${IPTABLES} -t nat -A PREROUTING -p ${PROTOCOL} -s ${BLOCK_SOURCE} -d ${BLOCK_DEST} --dport ${BLOCK_DEST_PORT} -j DROP
	;;
	*)
	echo "No Match"
	esac
	;;
\#*)
	# This is for commented lines
	;;
*)
	echo -e "\nInvalid syntax - ${SERVICETYPE}"
esac

done

############################################################
# End of new subroutine                                    #
############################################################

############################################################
# Default Rules                                            #
############################################################

#----------------------------------------------------------#
#     Default FORWARD policy: log and drop.
#----------------------------------------------------------#
#${IPTABLES} -A FORWARD -j LOG --log-prefix 'FORWARD policy [DROP]: '
${IPTABLES} -P FORWARD DROP

#----------------------------------------------------------#
#     Default OUTPUT policy: accept.
#----------------------------------------------------------#
${IPTABLES} -P OUTPUT ACCEPT

#----------------------------------------------------------#
#     Default INPUT policy: log and drop.
#----------------------------------------------------------#
#${IPTABLES} -A INPUT -j LOG --log-prefix 'INPUT policy [DROP]: '
${IPTABLES} -P INPUT DROP

############################################################
# Start other services                                     #
############################################################

#----------------------------------------------------------#
# Start freeswan
#----------------------------------------------------------#
case ${STARTIPSEC} in
yes|YES)
	echo -e "Now starting FreeS/WAN"
	${IPSEC} start
	;;
no|NO)
	echo "FreeS/WAN will not be started."
	;;
esac

#----------------------------------------------------------#
# Start squid
#----------------------------------------------------------#
case ${STARTSQUID} in
yes|YES)
	echo -e "Now starting Squid"
	${SQUID} start
	;;
no|NO)
	echo "Squid will not be started."
	;;
esac

############################################################
# The End                                                  #
############################################################
